﻿using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.Cells;
using Syncfusion.UI.Xaml.ScrollAxis;
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;

namespace DataGridTest2
{
    public class CustomGridCellTemplateRenderer : GridVirtualizingCellRenderer<ContentControl, ContentControl>
    {
        private readonly BindingHelper _bindingHelper;
        public CustomGridCellTemplateRenderer()
        {
            SupportsRenderOptimization = true;
            IsEditable = false;
            IsFocusible = true;
            _bindingHelper = new BindingHelper();
        }

        public override void OnInitializeDisplayElement(RowColumnIndex rowColumnIndex, ContentControl uiElement, GridColumn column, object dataContext)
        {
            var templateColumn = column as GridTemplateColumn;
            if (templateColumn != null)
                ApplyDisplayDataTemplate(uiElement, templateColumn, dataContext);
            uiElement.HorizontalContentAlignment = HorizontalAlignment.Stretch;
            uiElement.VerticalContentAlignment = VerticalAlignment.Stretch;
        }

        //public override void OnUpdateDisplayBinding(RowColumnIndex cellRowcolumnIndex, ContentControl uiElement, GridColumn column, object dataContext)
        //{
        //    var templateColumn = column as GridTemplateColumn;
        //    if (templateColumn != null)
        //        ApplyDisplayDataTemplate(uiElement, templateColumn, dataContext);
        //}

        protected override void InitializeCellStyle(RowColumnIndex cellRowColumnIndex, object record, UIElement cell, GridColumn column)
        {
            base.InitializeCellStyle(cellRowColumnIndex, record, cell, column);
            var content = GetValueFromDynamicObjects(record, column.MappingName);
            if (content == null) return;
            var gridCell = cell as GridCell;
            if (gridCell == null) return;
            var uiElement = (gridCell.Content as ContentControl);
            if (uiElement == null) return;

            var templateColumn = column as GridTemplateColumn;
            DataTemplate template = null;

            if (templateColumn.CellTemplateSelector != null)
                template = templateColumn.CellTemplateSelector.SelectTemplate(record, uiElement);

            if (template == null && templateColumn.CellTemplate != null)
                template = templateColumn.CellTemplate;

            if (template != null)
                uiElement.ContentTemplate = template;

            uiElement.DataContext = content;
            uiElement.Content = content;
        }

        private void ApplyDisplayDataTemplate(ContentControl uiElement, GridTemplateColumn templateColumn, object dataContext)
        {
            DataTemplate template = null;

            if (templateColumn.CellTemplateSelector != null)
                template = templateColumn.CellTemplateSelector.SelectTemplate(dataContext, uiElement);

            if (template == null && templateColumn.CellTemplate != null)
                template = templateColumn.CellTemplate;

            if (template != null)
                uiElement.ContentTemplate = template;

            if (!(dataContext is IDynamicMetaObjectProvider)) return;
            var content = GetValueFromDynamicObjects(dataContext, templateColumn.MappingName);
            //if (content == null)
            //{
            //    uiElement.DataContext = "";
            //    uiElement.Content = "";
            //    return;
            //}
            uiElement.DataContext = content;
            uiElement.Content = content;
        }

        private object GetValueFromDynamicObjects(object dataContext, string name)
        {
            _bindingHelper.DataContext = dataContext;
            _bindingHelper.SetBinding(BindingHelper.ValueProperty, new Binding { Path = new PropertyPath(name) });
            return _bindingHelper.Value == null ? "" : _bindingHelper.Value;
        }

        protected override void OnUnwireDisplayUIElement(ContentControl uiElement)
        {
            uiElement.ContentTemplate = null;
            base.OnUnwireDisplayUIElement(uiElement);
        }
    }

    class BindingHelper : FrameworkElement
    {
        public object Value
        {
            get { return GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        public static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", typeof(object), typeof(BindingHelper), new PropertyMetadata(null));

        public void SetValueBinding(Binding binding)
        {
            SetBinding(ValueProperty, binding);
        }
    }
}
