﻿using Syncfusion.Data;
using Syncfusion.UI.Xaml.Controls.DataPager;
using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.Cells;
using Syncfusion.UI.Xaml.ScrollAxis;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Dynamic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

namespace DataGridTest2
{
    public sealed partial class DataTable : UserControl
    {
        private Type _qrScannerType = null;
        private Type _scanDataType = null;
        private bool _isLookup = false;
        private bool _isChangingCondition = false;
        private bool _hasDataBeenSet = false;

        private SemaphoreSlim _filterChangedLock = new SemaphoreSlim(1);
        private SemaphoreSlim _lock = new SemaphoreSlim(1);


        public DataTable()
        {
            ResourceDict = Resources;
            Initialize();
        }

        #region Methods

       

        private void DataChanged()
        {
            _hasDataBeenSet = true;

            pager.PagedSource.SortDescriptions.Clear();
            pager.PagedSource.SortDescriptions.Add(new SortDescription
            {
                Direction = ListSortDirection.Descending,
                PropertyName = "ClientSubmitDate"
            });
        }

        private async void FilterConditionChanged()
        {
            await FilterChanged();
        }

        private async void FilterIndexChanged()
        {
            await FilterChanged();
        }

        private async void FilterStringChanged()
        {
            await FilterChanged();
        }

        private void Grid_CurrentCellActivated(object sender, CurrentCellActivatedEventArgs args)
        {

        }

        private void Grid_SelectionChanged(object sender, GridSelectionChangedEventArgs e)
        {

        }

        private void Pager_PageIndexChanging(object sender, Syncfusion.UI.Xaml.Controls.DataPager.PageIndexChangingEventArgs args)
        {
        }

        private void SelectedData_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {

        }

        private async void SortInfoChanged()
        {
            await FilterChanged();
        }

        #endregion

        #region ActionMethods
        private void Initialize()
        {
            this.InitializeComponent();

            pager.UseOnDemandPaging = true;

            grid.CellRenderers.Remove("Template");
            grid.CellRenderers.Add("Template", new CustomGridCellTemplateRenderer());


            grid.Columns.Clear();

            var column = new StringColumn();
            column.MappingName = "FirstName";

            grid.Columns.Add(column);

            var col = new GridTextColumn();
            col.MappingName = "LastName";

            grid.Columns.Add(col);
        }

        public async Task FilterChanged()
        {
                await _filterChangedLock.WaitAsync();

                if (pager.PagedSource != null)
                    pager.PagedSource.ResetCache();

                pager.MoveToFirstPage();

                if (pager.PagedSource != null && pager.PagedSource.Count > 0)
                    grid.ScrollInView(new Syncfusion.UI.Xaml.ScrollAxis.RowColumnIndex { ColumnIndex = 0, RowIndex = 0 });
         
        }

       
        #endregion

        #region EventHandlers (defined in Xaml)

        private void TxtText_TextChanged(object sender, TextChangedEventArgs e)
        {
            var textbox = sender as TextBox;

            if (textbox != null)
                FilterString = textbox.Text;
        }

        private void ColumnCombo_SelectionChanged(object sender, Windows.UI.Xaml.Controls.SelectionChangedEventArgs e)
        {
            var combobox = sender as ComboBox;

            if (combobox != null)
            {
                FilterIndex = combobox.SelectedIndex;
            }
        }

        private async void Pager_OnDemandLoading(object sender, Syncfusion.UI.Xaml.Controls.DataPager.OnDemandLoadingEventArgs args)
        {
            IEnumerable<DemoPerson> res = null;

            try
            {
                await _lock.WaitAsync();

                grid.Visibility = Visibility.Collapsed;
                pager.Visibility = Visibility.Collapsed;
                busyIndicator.IsActive = true;
                busyIndicator.Visibility = Visibility.Visible;

                switch (this.FilterIndex)
                {
                    case 1:
                        res = MainPage.PERSONS.Where(x => x.LastName.StartsWith(FilterString)).ToList();
                        break;

                    case 2:
                        res = MainPage.PERSONS.Where(x => x.Age.ToString().StartsWith(FilterString)).ToList();
                        break;

                    default:
                        res = MainPage.PERSONS.Where(x => x.FirstName.StartsWith(FilterString)).ToList();
                        break;
                }

                pager.PageCount = res.Count() / args.PageSize + (res.Count() % args.PageSize == 0 ? 0 : 1);
                pager.LoadDynamicItems(args.StartIndex, res);
                pager.PagedSource.Refresh();

                if (grid.Visibility == Windows.UI.Xaml.Visibility.Collapsed)
                {
                    grid.Visibility = Visibility.Visible;
                    pager.Visibility = Visibility.Visible;
                    busyIndicator.IsActive = false;
                    busyIndicator.Visibility = Visibility.Collapsed;
                }
            }
            finally
            {
                _lock.Release();
                _filterChangedLock.Release();
            }
        }

        #endregion

        #region Dependency Properties

        #region AccentBackground
        public SolidColorBrush AccentBackground
        {
            get { return (SolidColorBrush)GetValue(AccentBackgroundProperty); }
            set { SetValue(AccentBackgroundProperty, value); }
        }

        public static readonly DependencyProperty AccentBackgroundProperty =
            DependencyProperty.Register("AccentBackground", typeof(SolidColorBrush), typeof(DataTable), new PropertyMetadata(new SolidColorBrush()));
        #endregion

        #region AccentForeground
        public SolidColorBrush AccentForeground
        {
            get { return (SolidColorBrush)GetValue(AccentForegroundProperty); }
            set { SetValue(AccentForegroundProperty, value); }
        }

        public static readonly DependencyProperty AccentForegroundProperty =
            DependencyProperty.Register("AccentForeground", typeof(SolidColorBrush), typeof(DataTable), new PropertyMetadata(new SolidColorBrush()));
        #endregion     

        #region FilterIndex
        public int FilterIndex
        {
            get { return (int)GetValue(FilterIndexProperty); }
            set
            {
                SetValue(FilterIndexProperty, value);
                //if (columnCombo.SelectedIndex != FilterIndex + 2)
                //    columnCombo.SelectedIndex = FilterIndex + 2;
            }
        }

        public static readonly DependencyProperty FilterIndexProperty =
            DependencyProperty.Register("FilterIndex", typeof(int), typeof(DataTable), new PropertyMetadata(-2, new PropertyChangedCallback((DependencyObject o, DependencyPropertyChangedEventArgs e) =>
            {
                var dt = o as DataTable;

                if (dt != null)
                    dt.FilterIndexChanged();
            })));
        #endregion

        #region FilterString
        public string FilterString
        {
            get { return (string)GetValue(FilterStringProperty); }
            set { SetValue(FilterStringProperty, value); }
        }

        public static readonly DependencyProperty FilterStringProperty =
            DependencyProperty.Register("FilterString", typeof(string), typeof(DataTable), new PropertyMetadata(string.Empty, new PropertyChangedCallback((DependencyObject o, DependencyPropertyChangedEventArgs e) =>
            {
                var dt = o as DataTable;

                if (dt != null)
                    dt.FilterStringChanged();
            })));
        #endregion

        #endregion

        public static ResourceDictionary ResourceDict { get; set; }
    }

    public class StringColumn : GridTemplateColumn
    {
        public StringColumn()
            : base()
        {
            ColumnSizer = GridLengthUnitType.SizeToHeader;
            MinimumWidth = 100;
            CellTemplate = (DataTemplate)DataTable.ResourceDict["StringTemplate"];
        }
    }

    

}